Type: Package
Package: densityClust
Title: Clustering by Fast Search and Find of Density Peaks
Version: 0.3.3
Authors@R: c(
    person("Thomas Lin", "Pedersen", , "thomasp85@gmail.com", role = c("aut", "cre")),
    person("Sean", "Hughes", role = "aut"),
    person("Xiaojie", "Qiu", , "xqiu@uw.edu", role = "aut")
  )
Maintainer: Thomas Lin Pedersen <thomasp85@gmail.com>
Description: An improved implementation (based on k-nearest neighbors) of
    the density peak clustering algorithm, originally described by Alex
    Rodriguez and Alessandro Laio (Science, 2014 vol. 344). It can handle large 
    datasets (> 100,000 samples) very efficiently. It was initially implemented 
    by Thomas Lin Pedersen, with inputs from Sean Hughes and later improved by 
    Xiaojie Qiu to handle large datasets with kNNs.
License: GPL (>= 2)
URL: https://github.com/thomasp85/densityClust
BugReports: https://github.com/thomasp85/densityClust/issues
Imports: FNN, ggplot2, ggrepel, grDevices, gridExtra, RColorBrewer,
        Rtsne
Suggests: covr, testthat
LinkingTo: cpp11
Encoding: UTF-8
RoxygenNote: 7.3.1
NeedsCompilation: yes
Packaged: 2024-01-29 13:51:42 UTC; thomas
Author: Thomas Lin Pedersen [aut, cre],
  Sean Hughes [aut],
  Xiaojie Qiu [aut]
Repository: CRAN
Date/Publication: 2024-01-29 14:30:02 UTC
Built: R 4.2.3; aarch64-apple-darwin20; 2024-01-29 17:43:39 UTC; unix
Archs: densityClust.so.dSYM
