# Generated by roxygen2: do not edit by hand

S3method(clustered,densityCluster)
S3method(clusters,densityCluster)
S3method(findClusters,densityCluster)
S3method(labels,densityCluster)
S3method(plot,densityCluster)
S3method(plotMDS,densityCluster)
S3method(plotTSNE,densityCluster)
S3method(print,densityCluster)
export(clustered)
export(clusters)
export(densityClust)
export(estimateDc)
export(findClusters)
export(plotDensityClust)
export(plotMDS)
export(plotTSNE)
importFrom(FNN,get.knn)
importFrom(RColorBrewer,brewer.pal)
importFrom(Rtsne,Rtsne)
importFrom(ggplot2,aes_string)
importFrom(ggplot2,geom_label)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggrepel,geom_label_repel)
importFrom(grDevices,rainbow)
importFrom(graphics,legend)
importFrom(graphics,locator)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(gridExtra,grid.arrange)
importFrom(stats,cmdscale)
importFrom(stats,dist)
importFrom(stats,rnorm)
useDynLib(densityClust, .registration = TRUE)
