# Generated by cpp11: do not edit by hand

distanceToPeakCpp <- function(distance, rho) {
  .Call(`_densityClust_distanceToPeakCpp`, distance, rho)
}

findDistValueByRowColInd <- function(distance, num_row, row_inds, col_inds) {
  .Call(`_densityClust_findDistValueByRowColInd`, distance, num_row, row_inds, col_inds)
}

smallest_dist_rho_order_coords <- function(ordered_rho, ordered_coords) {
  .Call(`_densityClust_smallest_dist_rho_order_coords`, ordered_rho, ordered_coords)
}

gaussianLocalDensity <- function(distance, nrow, dc) {
  .Call(`_densityClust_gaussianLocalDensity`, distance, nrow, dc)
}

nonGaussianLocalDensity <- function(distance, nrow, dc) {
  .Call(`_densityClust_nonGaussianLocalDensity`, distance, nrow, dc)
}
